@@ -1,9 +1,9 @@
 """MCP Vector Search - CLI-first semantic code search with MCP integration."""
 
-__version__ = "0.15.12"
-__build__ = "83"
+__version__ = "1.1.7"
+__build__ = "107"
 __author__ = "Robert Matsuoka"
-__email__ = "bobmatnyc@gmail.com"
+__email__ = "bob@matsuoka.com"
 
 from .core.exceptions import MCPVectorSearchError
 
