@@ -1,5 +1,8 @@
 """Main CLI application for MCP Vector Search."""
 
+import faulthandler
+import signal
+import sys
 from pathlib import Path
 
 import typer
@@ -9,9 +12,52 @@ from rich.traceback import install
 
 from .. import __build__, __version__
 from .didyoumean import add_common_suggestions, create_enhanced_typer
-from .output import print_warning, setup_logging
+from .output import setup_logging
 from .suggestions import get_contextual_suggestions
 
+
+# ============================================================================
+# SIGNAL HANDLERS - Register early for crash diagnostics
+# ============================================================================
+def _handle_segfault(signum: int, frame) -> None:
+    """Handle segmentation faults with helpful error message.
+
+    Segmentation faults typically occur due to corrupted ChromaDB index data
+    or issues with native libraries (sentence-transformers, tree-sitter).
+
+    Args:
+        signum: Signal number (SIGSEGV = 11)
+        frame: Current stack frame (unused)
+    """
+    error_message = """
+╭─────────────────────────────────────────────────────────────────╮
+│ ⚠️  Segmentation Fault Detected                                  │
+├─────────────────────────────────────────────────────────────────┤
+│ This usually indicates corrupted index data or a crash in       │
+│ native libraries (ChromaDB, sentence-transformers, tree-sitter).│
+│                                                                 │
+│ To fix this, please run:                                        │
+│   1. mcp-vector-search reset index --force                      │
+│   2. mcp-vector-search index                                    │
+│                                                                 │
+│ This will rebuild your search index from scratch.               │
+│                                                                 │
+│ If the problem persists:                                        │
+│   - Try updating dependencies: pip install -U mcp-vector-search │
+│   - Check GitHub issues: github.com/bobmatnyc/mcp-vector-search │
+╰─────────────────────────────────────────────────────────────────╯
+"""
+    print(error_message, file=sys.stderr)
+    sys.exit(139)  # Standard segfault exit code (128 + 11)
+
+
+# Register signal handler for segmentation faults
+signal.signal(signal.SIGSEGV, _handle_segfault)
+
+# Enable faulthandler for better crash diagnostics
+# This prints Python traceback on segfaults before signal handler runs
+faulthandler.enable()
+
 # Install rich traceback handler
 install(show_locals=True)
 
@@ -57,6 +103,7 @@ explore unfamiliar projects, and integrate with AI coding tools via MCP.
   demo       🎬 Run interactive demo
   doctor     🩺 Check system health
   index      📇 Index codebase
+  reset      🔄 Reset and recovery operations
   mcp        🔌 MCP server operations
   config     ⚙️  Configure settings
   help       ❓ Get help
@@ -69,6 +116,7 @@ explore unfamiliar projects, and integrate with AI coding tools via MCP.
 )
 
 # Import command modules
+from .commands.analyze import analyze_app  # noqa: E402
 from .commands.chat import chat_app  # noqa: E402
 from .commands.config import config_app  # noqa: E402
 from .commands.demo import demo_app  # noqa: E402
@@ -76,6 +124,7 @@ from .commands.index import index_app  # noqa: E402
 from .commands.init import init_app  # noqa: E402
 from .commands.install import install_app  # noqa: E402
 from .commands.mcp import mcp_app  # noqa: E402
+from .commands.reset import reset_app  # noqa: E402
 from .commands.search import search_app, search_main  # noqa: E402, F401
 from .commands.setup import setup_app  # noqa: E402
 from .commands.status import main as status_main  # noqa: E402
@@ -127,152 +176,26 @@ app.add_typer(mcp_app, name="mcp", help="🔌 MCP server operations")
 # 10. CONFIG - Configuration
 app.add_typer(config_app, name="config", help="⚙️  Manage project configuration")
 
-# 11. VISUALIZE - Code graph visualization
+# 10.5. RESET - Reset and recovery operations
+app.add_typer(reset_app, name="reset", help="🔄 Reset and recovery operations")
+
+# 11. ANALYZE - Code complexity analysis
+app.add_typer(
+    analyze_app, name="analyze", help="📈 Analyze code complexity and quality"
+)
+
+# 12. VISUALIZE - Code graph visualization
 app.add_typer(
     visualize_app, name="visualize", help="📊 Visualize code chunk relationships"
 )
 
-# 12. HELP - Enhanced help
+# 13. HELP - Enhanced help
 # (defined below inline)
 
-# 13. VERSION - Version info
+# 14. VERSION - Version info
 # (defined below inline)
 
 
-# ============================================================================
-# DEPRECATED COMMANDS - With helpful suggestions
-# ============================================================================
-
-
-def _deprecated_command(old_cmd: str, new_cmd: str):
-    """Helper to create deprecated command with suggestion."""
-
-    def wrapper(*args, **kwargs):
-        print_warning(
-            f"⚠️  The command '{old_cmd}' is deprecated.\n"
-            f"   Please use '{new_cmd}' instead.\n"
-            f"   Run: [cyan]mcp-vector-search {new_cmd} --help[/cyan] for details."
-        )
-        raise typer.Exit(1)
-
-    return wrapper
-
-
-# NOTE: 'install' command is now the primary command for project installation
-# Old 'install' was deprecated in favor of 'init' in v0.7.0
-# Now 'install' is back as the hierarchical installation command in v0.13.0
-
-
-# Deprecated: find -> search
-@app.command("find", hidden=True)
-def deprecated_find():
-    """[DEPRECATED] Use 'search' instead."""
-    _deprecated_command("find", "search")()
-
-
-# Deprecated: search-similar -> search --similar
-@app.command("search-similar", hidden=True)
-def deprecated_search_similar():
-    """[DEPRECATED] Use 'search --similar' instead."""
-    _deprecated_command("search-similar", "search --similar")()
-
-
-# Deprecated: search-context -> search --context
-@app.command("search-context", hidden=True)
-def deprecated_search_context():
-    """[DEPRECATED] Use 'search --context' instead."""
-    _deprecated_command("search-context", "search --context")()
-
-
-# Deprecated: interactive -> search interactive
-@app.command("interactive", hidden=True)
-def deprecated_interactive():
-    """[DEPRECATED] Use 'search interactive' instead."""
-    _deprecated_command("interactive", "search interactive")()
-
-
-# Deprecated: history -> search history
-@app.command("history", hidden=True)
-def deprecated_history():
-    """[DEPRECATED] Use 'search history' instead."""
-    _deprecated_command("history", "search history")()
-
-
-# Deprecated: favorites -> search favorites
-@app.command("favorites", hidden=True)
-def deprecated_favorites():
-    """[DEPRECATED] Use 'search favorites' instead."""
-    _deprecated_command("favorites", "search favorites")()
-
-
-# Deprecated: add-favorite -> search favorites add
-@app.command("add-favorite", hidden=True)
-def deprecated_add_favorite():
-    """[DEPRECATED] Use 'search favorites add' instead."""
-    _deprecated_command("add-favorite", "search favorites add")()
-
-
-# Deprecated: remove-favorite -> search favorites remove
-@app.command("remove-favorite", hidden=True)
-def deprecated_remove_favorite():
-    """[DEPRECATED] Use 'search favorites remove' instead."""
-    _deprecated_command("remove-favorite", "search favorites remove")()
-
-
-# Deprecated: health -> index health
-@app.command("health", hidden=True)
-def deprecated_health():
-    """[DEPRECATED] Use 'index health' instead."""
-    _deprecated_command("health", "index health")()
-
-
-# Deprecated: watch -> index watch
-@app.command("watch", hidden=True)
-def deprecated_watch():
-    """[DEPRECATED] Use 'index watch' instead."""
-    _deprecated_command("watch", "index watch")()
-
-
-# Deprecated: auto-index -> index auto
-@app.command("auto-index", hidden=True)
-def deprecated_auto_index():
-    """[DEPRECATED] Use 'index auto' instead."""
-    _deprecated_command("auto-index", "index auto")()
-
-
-# Deprecated: reset -> mcp reset or config reset
-@app.command("reset", hidden=True)
-def deprecated_reset():
-    """[DEPRECATED] Use 'mcp reset' or 'config reset' instead."""
-    print_warning(
-        "⚠️  The 'reset' command is deprecated.\n"
-        "   Use [cyan]mcp-vector-search mcp reset[/cyan] for MCP reset\n"
-        "   Use [cyan]mcp-vector-search config reset[/cyan] for config reset"
-    )
-    raise typer.Exit(1)
-
-
-# Deprecated: init-check -> init check
-@app.command("init-check", hidden=True)
-def deprecated_init_check():
-    """[DEPRECATED] Use 'init check' instead."""
-    _deprecated_command("init-check", "init check")()
-
-
-# Deprecated: init-mcp -> mcp install
-@app.command("init-mcp", hidden=True)
-def deprecated_init_mcp():
-    """[DEPRECATED] Use 'mcp install' instead."""
-    _deprecated_command("init-mcp", "mcp install")()
-
-
-# Deprecated: init-models -> config models
-@app.command("init-models", hidden=True)
-def deprecated_init_models():
-    """[DEPRECATED] Use 'config models' instead."""
-    _deprecated_command("init-models", "config models")()
-
-
 # ============================================================================
 # MAIN INLINE COMMANDS
 # ============================================================================
@@ -434,7 +357,11 @@ def cli_with_suggestions():
 
     try:
         # Call the app with standalone_mode=False to get exceptions instead of sys.exit
-        app(standalone_mode=False)
+        # Capture return value - when standalone_mode=False, typer.Exit returns code instead of raising
+        exit_code = app(standalone_mode=False)
+        # Propagate non-zero exit codes (e.g., from --fail-on-smell quality gate)
+        if exit_code is not None and exit_code != 0:
+            sys.exit(exit_code)
     except click.UsageError as e:
         # Check if it's a "No such command" error
         if "No such command" in str(e):
@@ -472,8 +399,12 @@ def cli_with_suggestions():
     except click.Abort:
         # User interrupted (Ctrl+C)
         sys.exit(1)
-    except (SystemExit, click.exceptions.Exit):
-        # Re-raise system exits and typer.Exit
+    except (SystemExit, click.exceptions.Exit) as e:
+        # Re-raise system exits and typer.Exit with their exit codes
+        if hasattr(e, "exit_code"):
+            sys.exit(e.exit_code)
+        elif hasattr(e, "code"):
+            sys.exit(e.code if e.code is not None else 0)
         raise
     except Exception as e:
         # For other exceptions, show error and exit if verbose logging is enabled
